==> python-tensorflow-cuda_1.9.0.orig.tar.gz: tensorflow-1.9.0/tensorflow/contrib/lite/toco/graph_transformations/propagate_default_min_max.cc <==
/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================*/
#include <memory>
#include <string>
#include <unordered_map>
#include <vector>

#include "tensorflow/contrib/lite/toco/graph_transformations/graph_transformations.h"
#include "tensorflow/contrib/lite/toco/graph_transformations/quantization_util.h"
#include "tensorflow/contrib/lite/toco/model.h"
#include "tensorflow/contrib/lite/toco/tooling_util.h"
#include "tensorflow/core/platform/logging.h"

namespace toco {

// Propagates default min/max values to any operator input/output array that
// is missing them.
//
// When provided a set of min/max values for uint8 arrays this will rescale
// the values for other data types as required and preserving the floating point
// range within the new type.
bool PropagateDefaultMinMax::Run(Model* model, std::size_t op_index) {
  const auto it = model->operators.begin() + op_index;
  const auto* op = it->get();

  bool did_change = false;

  for (const auto& input : op->inputs) {
    auto& input_array = model->GetArray(input);
    if (!input_array.minmax && !input_array.buffer) {
      did_change |= SetArrayMinMax(input, &input_array);
    }
  }

  for (const auto& output : op->outputs) {
    auto& output_array = model->GetArray(output);
    if (!output_array.minmax && !output_array.buffer) {
      did_change |= SetArrayMinMax(output, &output_array);
    }
  }

  return did_change;
}

// Sets the min/max on the given array, adjusting the reference_minmax for the
// final data type of the array if it is already specified.
bool PropagateDefaultMinMax::SetArrayMinMax(const string& array_name,
                                            Array* array) {
  CHECK(!array->minmax);

  ArrayDataType quantized_data_type =
      GetQuantizedDataType(*array, ArrayDataType::kUint8);
  for (const auto& type_range : type_ranges_) {
    if (type_range.first == quantized_data_type) {
      array->GetOrCreateMinMax() = type_range.second;
      break;
    }
  }
  if (!array->minmax) {
    AddMessageF(
        "No defaults specified for quantized data type %s of array %s, "
        "skipping",
        ArrayDataTypeName(quantized_data_type), array_name);
    return false;
  }

  AddMessageF("Adding default minmax %g,%g to array %s when quantized as %s",
              array->GetMinMax().min, array->GetMinMax().max, array_name,
              ArrayDataTypeName(quantized_data_type));

  return true;
}

}  // namespace toco

==> python-tensorflow-cuda_1.12.0.orig.tar.gz: tensorflow-1.12.0/tensorflow/contrib/lite/toco/graph_transformations/propagate_default_min_max.cc <==
/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================*/
#include <memory>
#include <string>
#include <unordered_map>
#include <vector>

#include "tensorflow/contrib/lite/toco/graph_transformations/graph_transformations.h"
#include "tensorflow/contrib/lite/toco/graph_transformations/quantization_util.h"
#include "tensorflow/contrib/lite/toco/model.h"
#include "tensorflow/contrib/lite/toco/tooling_util.h"
#include "tensorflow/core/platform/logging.h"

namespace toco {

namespace {

bool SupportsMinMax(const Array& array) {
  return array.data_type == ArrayDataType::kFloat;
}

}  // namespace

// Propagates default min/max values to any operator input/output array that
// is missing them.
//
// When provided a set of min/max values for uint8 arrays this will rescale
// the values for other data types as required and preserving the floating point
// range within the new type.
bool PropagateDefaultMinMax::Run(Model* model, std::size_t op_index) {
  const auto it = model->operators.begin() + op_index;
  const auto* op = it->get();

  bool did_change = false;

  for (const auto& input : op->inputs) {
    auto& input_array = model->GetArray(input);
    if (!input_array.minmax && !input_array.buffer &&
        SupportsMinMax(input_array)) {
      did_change |= SetArrayMinMax(input, &input_array);
    }
  }

  for (const auto& output : op->outputs) {
    auto& output_array = model->GetArray(output);
    if (!output_array.minmax && !output_array.buffer &&
        SupportsMinMax(output_array)) {
      did_change |= SetArrayMinMax(output, &output_array);
    }
  }

  return did_change;
}

// Sets the min/max on the given array, adjusting the reference_minmax for the
// final data type of the array if it is already specified.
bool PropagateDefaultMinMax::SetArrayMinMax(const string& array_name,
                                            Array* array) {
  CHECK(!array->minmax);

  ArrayDataType quantized_data_type =
      GetQuantizedDataType(*array, ArrayDataType::kUint8);
  for (const auto& type_range : type_ranges_) {
    if (type_range.first == quantized_data_type) {
      array->GetOrCreateMinMax() = type_range.second;
      break;
    }
  }
  if (!array->minmax) {
    AddMessageF(
        "No defaults specified for quantized data type %s of array %s, "
        "skipping",
        ArrayDataTypeName(quantized_data_type), array_name);
    return false;
  }

  AddMessageF("Adding default minmax %g,%g to array %s when quantized as %s",
              array->GetMinMax().min, array->GetMinMax().max, array_name,
              ArrayDataTypeName(quantized_data_type));

  return true;
}

}  // namespace toco
